import org.hamcrest.CoreMatchers;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;
import org.powermock.reflect.Whitebox;

import static org.junit.Assert.assertThat;
import static org.mockito.Matchers.anyString;
import static org.mockito.internal.verification.VerificationModeFactory.times;
import static org.powermock.api.mockito.PowerMockito.spy;
import static org.powermock.api.mockito.PowerMockito.when;
import static org.powermock.api.support.membermodification.MemberMatcher.method;

@RunWith(PowerMockRunner.class)
@PrepareForTest(TestSubjectClass.class)
public class TestSubjectClassTest {

    @Test
    public void testPrivateMethod() throws Exception {
        TestSubjectClass testedClass = new TestSubjectClass();

        String actual = Whitebox.invokeMethod(testedClass,
                "internalLogic", "Test method input value");

        assertThat(actual, CoreMatchers.containsString("Test method input value"));
    }

    @Test
    public void mockedPrivateMethod() throws Exception {
        TestSubjectClass testedClassSpy = spy(new TestSubjectClass());
        when(testedClassSpy, method(TestSubjectClass.class, "internalLogic")).withArguments(anyString()).thenReturn("TestInternalLogicOutcome");
        String output = testedClassSpy.doSomething();


        PowerMockito.verifyPrivate(testedClassSpy, times(1)).invoke("internalLogic", "1");
        assertThat(output, CoreMatchers.containsString("TestInternalLogicOutcome"));

    }

    @Test
    public void mockNewObjectCreation() throws Exception {
        TestSubjectClass testedClass = new TestSubjectClass();
        Integer mockedValue = new Integer(5);

        PowerMockito.whenNew(Integer.class)
                .withAnyArguments().thenReturn(mockedValue);

        String output = testedClass.doSomething();

        assertThat(output, CoreMatchers.containsString("Here is an input: 5"));

    }
}

